@@ -35,7 +35,7 @@ ncmpio_create(MPI_Comm     comm,
               const char  *path,
               int          cmode,
               int          ncid,
-              MPI_Info     info, /* user's info and env info combined */
+              MPI_Info     user_info, /* user's and env info combined */
               void       **ncpp)
 {
     char *env_str;
@@ -51,33 +51,19 @@ ncmpio_create(MPI_Comm     comm,
      * path consistency will be done in MPI_File_open */
 
     /* First, check whether cmode is valid or supported ---------------------*/
+
     /* NC_DISKLESS is not supported yet */
     if (cmode & NC_DISKLESS) DEBUG_RETURN_ERROR(NC_EINVAL_CMODE)
 
     /* NC_MMAP is not supported yet */
     if (cmode & NC_MMAP) DEBUG_RETURN_ERROR(NC_EINVAL_CMODE)
 
+    /* Check cmode for other illegal flags already done in dispatcher layer */
+
     /* Get default format, in case cmode does not include either
      * NC_64BIT_OFFSET or NC_64BIT_DATA */
     ncmpi_inq_default_format(&default_format);
 
-#if SIZEOF_MPI_OFFSET <  8
-    /* check cmode */
-    if (fIsSet(cmode, NC_64BIT_DATA)     || fIsSet(cmode, NC_64BIT_OFFSET)   ||
-        default_format == NC_FORMAT_CDF5 || default_format == NC_FORMAT_CDF2) {
-        /* unlike serial netcdf, we will not bother to support
-         * NC_64BIT_OFFSET on systems with off_t smaller than 8 bytes.
-         * serial netcdf has proven it's possible if datasets are small, but
-         * that's a hassle we don't want to worry about */
-        DEBUG_RETURN_ERROR(NC_ESMALL)
-    }
-#endif
-
-    /* It is illegal to have both NC_64BIT_OFFSET & NC_64BIT_DATA */
-    if ((cmode & (NC_64BIT_OFFSET|NC_64BIT_DATA)) ==
-                 (NC_64BIT_OFFSET|NC_64BIT_DATA))
-        DEBUG_RETURN_ERROR(NC_EINVAL_CMODE)
-
     /* Handle file clobber --------------------------------------------------*/
     MPI_Comm_rank(comm, &rank);
 
@@ -143,7 +129,7 @@ ncmpio_create(MPI_Comm     comm,
     }
 
     /* create file collectively -------------------------------------------- */
-    TRACE_IO(MPI_File_open)(comm, (char *)path, mpiomode, info, &fh);
+    TRACE_IO(MPI_File_open)(comm, (char *)path, mpiomode, user_info, &fh);
     if (mpireturn != MPI_SUCCESS) {
 #ifndef HAVE_ACCESS
         if (fIsSet(cmode, NC_NOCLOBBER)) {
@@ -171,7 +157,7 @@ ncmpio_create(MPI_Comm     comm,
         /* reset errno, as MPI_File_open may change it, even for MPI_SUCCESS */
         errno = 0;
 
-    /* get the file info actually used by MPI-IO (may alter user's info) */
+    /* get the I/O hints used/modified by MPI-IO */
     mpireturn = MPI_File_get_info(fh, &info_used);
     if (mpireturn != MPI_SUCCESS)
         return ncmpii_error_mpi2nc(mpireturn, "MPI_File_get_info");
@@ -210,13 +196,20 @@ ncmpio_create(MPI_Comm     comm,
     /* initialize unlimited_id as no unlimited dimension yet defined */
     ncp->dims.unlimited_id = -1;
 
-    /* extract I/O hints from user info */
-    ncmpio_set_pnetcdf_hints(ncp, info);
+    /* buffer to pack noncontiguous user buffers when calling wait() */
+    ncp->ibuf_size = NC_DEFAULT_IBUF_SIZE;
+
+    /* Extract PnetCDF specific I/O hints from user_info and set default hint
+     * values into info_used. Note some MPI libraries, such as MPICH 3.3.1 and
+     * priors fail to preserve user hints that are not recogniozed by the MPI
+     * libraries.
+     */
+    ncmpio_set_pnetcdf_hints(ncp, user_info, info_used);
 
     /* For file create, ignore if NC_NOWRITE set in cmode by user */
     ncp->iomode         = cmode | NC_WRITE;
     ncp->comm           = comm;  /* reuse comm duplicated in dispatch layer */
-    ncp->mpiinfo        = info_used;
+    ncp->mpiinfo        = info_used; /* is not MPI_INFO_NULL */
     ncp->mpiomode       = mpiomode;
     ncp->collective_fh  = fh;
     ncp->independent_fh = MPI_FILE_NULL;
