@@ -14,7 +14,7 @@
  * ncmpi_end_indep_data()   : dispatcher->end_indep_data()
  * ncmpi_inq()              : dispatcher->inq()
  * ncmpi_inq_xxx()          : dispatcher->inq_misc()
- * ncmpi_flush()             : dispatcher->flush()
+ * ncmpi_flush()            : dispatcher->flush()
  */
 
 #ifdef HAVE_CONFIG_H
@@ -308,12 +308,13 @@ ncmpio_inq_misc(void       *ncdp,
 
     /* Get the file pathname which was used to open/create the ncid's file.
      * path must already be allocated. Ignored if NULL */
-    if (ncp->path == NULL) {
-        if (pathlen != NULL) *pathlen = 0;
-        if (path    != NULL) *path = '\0';
-    } else {
-        if (pathlen != NULL) *pathlen = (int)strlen(ncp->path);
-        if (path    != NULL) strcpy(path, ncp->path);
+    if (pathlen != NULL) {
+        if (ncp->path == NULL) *pathlen = 0;
+        else                   *pathlen = (int)strlen(ncp->path);
+    }
+    if (path != NULL) {
+        if (ncp->path == NULL) *path = '\0';
+        else                   strcpy(path, ncp->path);
     }
 
     /* obtain the number of fixed-size variables */
@@ -384,11 +385,21 @@ ncmpio_inq_misc(void       *ncdp,
         mpireturn = MPI_Info_dup(ncp->mpiinfo, info_used);
         if (mpireturn != MPI_SUCCESS)
             return ncmpii_error_mpi2nc(mpireturn, "MPI_Info_dup");
+#if 0
+        /* PnetCDF hints have been added to ncp->mpiinfo at ncmpi_enddef.
+         *
+         * Note some MPI libraries, such as MPICH 3.3.1 and prior, fail to
+         * preserve user hints that are not recognized by the MPI libraries,
+         * which can result in the PnetCDF hints undefined in the info object
+         * returend from MPI_File_get_info(). We need to check if this is the
+         * case and add the PnbetCDF hints explicitly to the info object
+         * brefore returning it to user.
+         */
 
         sprintf(value, "%lld", ncp->h_align);
         MPI_Info_set(*info_used, "nc_header_align_size", value);
 
-        sprintf(value, "%lld", ncp->v_align);
+        sprintf(value, "%lld", ncp->fx_v_align);
         MPI_Info_set(*info_used, "nc_var_align_size", value);
 
         sprintf(value, "%lld", ncp->r_align);
@@ -404,6 +415,9 @@ ncmpio_inq_misc(void       *ncdp,
         else
             MPI_Info_set(*info_used, "nc_in_place_swap", "auto");
 
+        sprintf(value, "%lld", ncp->ibuf_size);
+        MPI_Info_set(*info_used, "nc_ibuf_size", value);
+
 #ifdef ENABLE_SUBFILING
         if (ncp->subfile_mode)
             MPI_Info_set(*info_used, "pnetcdf_subfiling", "enable");
@@ -413,6 +427,7 @@ ncmpio_inq_misc(void       *ncdp,
         MPI_Info_set(*info_used, "nc_num_subfiles", value);
 #else
         MPI_Info_set(*info_used, "pnetcdf_subfiling", "disable");
+#endif
 #endif
     }
 
