@@ -23,6 +23,16 @@
 #include "dumplib.h"
 #include "vardata.h"
 
+#ifdef ENABLE_ADIOS
+#include "adios_read.h"
+#include <arpa/inet.h>
+#define BP_MINIFOOTER_SIZE 28
+#define ADIOS_VERSION_NUM_MASK                       0x000000FF
+#define BUFREAD64(buf,var) var = *(off_t *) (buf); \
+                         if (diff_endian) \
+                             swap_64(&var);
+#endif
+
 static void usage(void);
 static char* name_path(const char* path);
 static const char* type_name(nc_type  type);
@@ -40,6 +50,10 @@ int main(int argc, char** argv);
 
 char *progname;
 
+#ifdef ENABLE_ADIOS
+unsigned int bp_ver;
+#endif
+
 static void
 usage(void)
 {
@@ -52,7 +66,7 @@ usage(void)
   [-l len]         Line length maximum in data section (default 80)\n\
   [-n name]        Name for netCDF (default derived from file name)\n\
   [-p n[,n]]       Display floating-point values with less precision\n\
-  [-V]             Print the file format (CDF-1, CDF-2, or CDF-5)\n\
+  [-V]             Print the file format (CDF-1, CDF-2, CDF-5, or NetCDF-4)\n\
   [-k]             Print kind of file (classic, 64-bit offset, or 64-bit data)\n\
   file             File name of input netCDF file\n"
 
@@ -393,6 +407,14 @@ do_ncdump(const char *path, struct fspec* specp)
             Printf ("%s file format: CDF-5 (big variables)\n", specp->name);
         else if (NC_mode == NC_64BIT_OFFSET)
             Printf ("%s file format: CDF-2 (large file)\n", specp->name);
+#ifdef ENABLE_NETCDF4
+        else if (NC_mode == NC_NETCDF4)
+            Printf ("%s file format: NetCDF-4\n", specp->name);
+#endif
+#ifdef ENABLE_ADIOS
+        else if (NC_mode == NC_BP)
+            Printf ("%s file format: ADIOS BP Ver. %u\n", specp->name, bp_ver);
+#endif
         else
             Printf ("%s file format: CDF-1\n", specp->name);
     } else if (specp->kind) {
@@ -409,6 +431,14 @@ do_ncdump(const char *path, struct fspec* specp)
             Printf ("// file format: CDF-5 (big variables)\n");
         else if (NC_mode == NC_64BIT_OFFSET)
             Printf ("// file format: CDF-2 (large file)\n");
+#ifdef ENABLE_NETCDF4
+        else if (NC_mode == NC_NETCDF4)
+            Printf ("// file format: NetCDF-4\n");
+#endif
+#ifdef ENABLE_ADIOS
+        else if (NC_mode == NC_BP)
+            Printf ("// file format: ADIOS BP Ver. %u\n", bp_ver);
+#endif
         else
             Printf ("// file format: CDF-1\n");
         /*
@@ -672,9 +702,44 @@ enum FILE_KIND {
     CDF2,
     CDF1,
     HDF5,
+    BP,
     UNKNOWN
 };
 
+#ifdef ENABLE_ADIOS
+static void swap_64(void *data)
+{
+    uint64_t d = *(uint64_t *)data;
+    *(uint64_t *)data = ((d&0x00000000000000FF)<<56)
+                          + ((d&0x000000000000FF00)<<40)
+                          + ((d&0x0000000000FF0000)<<24)
+                          + ((d&0x00000000FF000000)<<8)
+                          + ((d&0x000000FF00000000LL)>>8)
+                          + ((d&0x0000FF0000000000LL)>>24)
+                          + ((d&0x00FF000000000000LL)>>40)
+                          + ((d&0xFF00000000000000LL)>>56);
+}
+
+static int adios_parse_version (char *footer, unsigned int *version,
+                                int *diff_endianness) {
+    unsigned int test = 1; /* If high bit set, big endian */
+
+    *version = ntohl (*(uint32_t *) (footer + BP_MINIFOOTER_SIZE - 4));
+    char *v = (char *) version;
+    if ((*v && !*(char *) &test) /* Both writer and reader are big endian */
+        || (!*(v+3) && *(char *) &test)){ /* Both are little endian */
+        *diff_endianness = 0; /* No need to change endiannness */
+    }
+    else{
+        *diff_endianness = 1;
+    }
+
+    *version = *version & 0x7fffffff;
+
+    return 0;
+}
+#endif
+
 static
 enum FILE_KIND check_file_signature(char *path)
 {
@@ -708,6 +773,55 @@ enum FILE_KIND check_file_signature(char *path)
         else if (signature[3] == 2)  return CDF2;
         else if (signature[3] == 1)  return CDF1;
     }
+#ifdef ENABLE_ADIOS
+    else{
+        off_t fsize;
+        int diff_endian;
+        char footer[BP_MINIFOOTER_SIZE];
+        off_t h1, h2, h3;
+
+        if ((fd = open(path, O_RDONLY, 0700)) == -1) {
+            fprintf(stderr,"%s error at opening file %s (%s)\n",progname,path,strerror(errno));
+            return UNKNOWN;
+        }
+
+        /* Seek to footer */
+        fsize = lseek(fd, (off_t)(-(BP_MINIFOOTER_SIZE)), SEEK_END);
+
+        /* Get footer */
+        rlen = read(fd, footer, BP_MINIFOOTER_SIZE);
+        if (rlen != BP_MINIFOOTER_SIZE) {
+            if (rlen < 0)
+                fprintf(stderr,"%s error at reading file %s (%s)\n",progname,path,strerror(errno));
+            else
+                fprintf(stderr,"%s error: unknown file format\n",progname);
+            close(fd); /* ignore error */
+            return UNKNOWN;
+        }
+        if (close(fd) == -1) {
+            fprintf(stderr,"%s error at closing file %s (%s)\n",progname,path,strerror(errno));
+            return UNKNOWN;
+        }
+
+        adios_parse_version(footer, &bp_ver, &diff_endian);
+        bp_ver = bp_ver & ADIOS_VERSION_NUM_MASK;
+
+        BUFREAD64(footer, h1) /* Position of process group index table */
+        BUFREAD64(footer + 8, h2) /* Position of variables index table */
+        BUFREAD64(footer + 16, h3) /* Position of attributes index table */
+
+        /* All index tables must fall within the file
+         * Process group index table must comes before variable index table.
+         * Variables index table must comes before attributes index table.
+         */
+        if (0 < h1 && h1 < fsize &&
+            0 < h2 && h2 < fsize &&
+            0 < h3 && h3 < fsize &&
+            h1 < h2 && h2 < h3){
+            return BP;
+        }
+    }
+#endif
 
     return UNKNOWN; /* unknown format */
 }
@@ -870,13 +984,16 @@ main(int argc, char *argv[])
         else if (file_kind == CDF2) Printf ("64-bit offset\n");
         else if (file_kind == CDF1) Printf ("classic\n");
         else if (file_kind == HDF5) Printf ("NetCDF-4\n");
+        else if (file_kind == BP)   Printf ("ADIOS BP\n");
         goto fn_exit;
     }
+#ifndef ENABLE_NETCDF4
     else if (file_kind == HDF5) {
         if (rank == 0) fprintf(stderr,"%s error: file %s is an HDF5 file. Please use ncdump instead.\n",progname,argv[0]);
         err = EXIT_FAILURE;
         goto fn_exit; /* exit if is HDF5 */
     }
+#endif
 
     if (!nameopt) fspec.name = (char *)0;
     do_ncdump(argv[0], &fspec);
@@ -905,13 +1022,16 @@ main(int argc, char *argv[])
             else if (file_kind == CDF2) Printf ("64-bit offset\n");
             else if (file_kind == CDF1) Printf ("classic\n");
             else if (file_kind == HDF5) Printf ("NetCDF-4\n");
+            else if (file_kind == BP)   Printf ("ADIOS BP\n");
             goto fn_exit;
         }
+#ifndef ENABLE_NETCDF4
         else if (file_kind == HDF5) {
             if (rank == 0) fprintf(stderr,"%s error: file %s is an HDF5 file. Please use ncdump instead.\n",progname,argv[i]);
             err = EXIT_FAILURE;
             goto fn_exit; /* exit if is HDF5 */
         }
+#endif
 
         if (!nameopt) fspec.name = (char *)0;
         if (argc > 0)
