@@ -9,11 +9,11 @@
 #include <sys/types.h>  /* open() */
 #include <sys/stat.h>   /* open() */
 #include <fcntl.h>      /* open() */
-#include <unistd.h>     /* read() getopt() */
+#include <unistd.h>     /* read(), getopt() */
 #include <string.h>     /* strcpy(), strncpy() */
 #include <inttypes.h>   /* check for Endianness, uint32_t*/
 #include <assert.h>
-#include <errno.h>
+#include <errno.h>      /* errno */
 
 #define X_ALIGN         4
 #define X_INT_MAX       2147483647
@@ -45,7 +45,7 @@ static const char nada[4] = {0, 0, 0, 0};
 /* useful for aligning memory */
 #define _RNDUP(x, unit) ((((x) + (unit) - 1) / (unit)) * (unit))
 
-#define ERR_ADDR (((size_t) gbp->pos - (size_t) gbp->base) + gbp->offset - gbp->size)
+#define ERR_ADDR (((size_t)gbp->pos - (size_t)gbp->base) + (size_t)(gbp->offset - gbp->size))
 
 #define IS_RECVAR(vp) ((vp)->shape != NULL ? (*(vp)->shape == NC_UNLIMITED) : 0 )
 
@@ -171,7 +171,7 @@ typedef struct bufferinfo {
     int        version;  /* 1, 2, and 5 for CDF-1, 2, and 5 respectively */
     void      *base;     /* beginning of read/write buffer */
     void      *pos;      /* current position in buffer */
-    long long  size;     /* size of the buffer */
+    size_t     size;     /* size of the buffer */
 } bufferinfo;
 
 #define NC_NOERR        0       /**< No Error */
@@ -212,7 +212,7 @@ static const char ncmagic[] = {'C', 'D', 'F', 0x01};
 
 static int check_little_endian(void)
 {
-    // return 0 for big endian, 1 for little endian.
+    /* return 0 for big endian, 1 for little endian */
     volatile uint32_t i=0x01234567;
     return (*((uint8_t*)(&i))) == 0x67;
 }
@@ -339,8 +339,8 @@ free_NC_vararray(NC_vararray *ncap)
 
 /*----< hdr_len_NC_name() >--------------------------------------------------*/
 static long long
-hdr_len_NC_name(int nchars,
-                int sizeof_t)     /* NON_NEG */
+hdr_len_NC_name(size_t nchars,
+                int    sizeof_t)     /* NON_NEG */
 {
     /* netCDF file format:
      * name       = nelems  namestring
@@ -512,7 +512,7 @@ hdr_len_NC_var(const NC_var *varp,
      */
     sz = hdr_len_NC_name(varp->name_len, sizeof_t);     /* name */
     sz += sizeof_t;                                     /* nelems */
-    sz += sizeof_t * varp->ndims;                       /* [dimid ...] */
+    sz += (long long)sizeof_t * varp->ndims;            /* [dimid ...] */
     sz += hdr_len_NC_attrarray(&varp->attrs, sizeof_t); /* vatt_list */
     sz += 4;                                            /* nc_type */
     sz += sizeof_t;                                     /* vsize */
@@ -1031,6 +1031,8 @@ hdr_get_name(int          fd,
         if (err != NC_NOERR) {
             if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\t%s - fetching name string padding\n", loc);
+            free(*namep);
+            *namep = NULL;
             return err;
         }
         memset(pad, 0, X_ALIGN-1);
@@ -1038,8 +1040,6 @@ hdr_get_name(int          fd,
             /* This is considered not a fatal error, we continue to validate */
             if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\t%s \"%s\": name padding is non-null byte\n", loc, *namep);
-            // free(*namep);
-            // *namep = NULL;
             DEBUG_ASSIGN_ERROR(err, NC_ENULLPAD)
             if (repair) {
                 val_repair(fd, err_addr, (size_t)padding, (void*)nada);
@@ -1159,7 +1159,7 @@ val_get_NC_dimarray(int fd, bufferinfo *gbp, NC_dimarray *ncap, long long numrec
         return NC_NOERR;
 #if 0
         if (tag != ABSENT) {
-            if (verbose) printf("Error @ [0x%8.8Lx]:\n", err_addr);
+            if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\tInvalid NC component tag, while ABSENT is expected for ");
             DEBUG_RETURN_ERROR(NC_ENOTNC)
         }
@@ -1291,7 +1291,7 @@ val_get_NC_attrV(int         fd,
         memset(pad, 0, X_ALIGN-1);
         if (memcmp(gbp->pos, pad, padding) != 0) {
             /* This is considered not a fatal error, we continue to validate */
-            if (verbose) printf("Error @ [0x%8.8Lx]:\n", ERR_ADDR);
+            if (verbose) printf("Error @ [0x%8.8zx]:\n", ERR_ADDR);
             if (verbose) printf("\t%s: value padding is non-null byte\n", loc);
             DEBUG_ASSIGN_ERROR(status, NC_ENULLPAD)
             if (repair) {
@@ -1499,7 +1499,7 @@ val_get_NC_attrarray(int           fd,
         return NC_NOERR;
 #if 0
         if (tag != ABSENT) {
-            if (verbose) printf("Error @ [0x%8.8Lx]:\n", err_addr);
+            if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\tInvalid NC component tag, while ABSENT is expected for ");
             DEBUG_RETURN_ERROR(NC_ENOTNC)
         }
@@ -1663,7 +1663,7 @@ val_get_NC_var(int          fd,
         err = val_check_buffer(fd, gbp, (gbp->version < 5 ? 4 : 8));
         if (err != NC_NOERR) {
             if (trace) printf("\n");
-            if (verbose) printf("Error @ [0x%8.8Lx]:\n", ERR_ADDR);
+            if (verbose) printf("Error @ [0x%8.8zx]:\n", ERR_ADDR);
             if (verbose) printf("\t%s: Fail to read dimid[%d]\n",xloc,dim);
             free_NC_var(varp);
             return err;
@@ -1725,7 +1725,7 @@ val_get_NC_var(int          fd,
 
     err = val_check_buffer(fd, gbp, (gbp->version == 1 ? 4 : 8));
     if (err != NC_NOERR) {
-        if (verbose) printf("Error @ [0x%8.8Lx]:\n", ERR_ADDR);
+        if (verbose) printf("Error @ [0x%8.8zx]:\n", ERR_ADDR);
         if (verbose) printf("\t%s: Fail to read begin\n",xloc);
         free_NC_var(varp);
         return err;
@@ -1805,7 +1805,7 @@ val_get_NC_vararray(int          fd,
         return NC_NOERR;
 #if 0
         if (tag != ABSENT) {
-            if (verbose) printf("Error @ [0x%8.8Lx]:\n", err_addr);
+            if (verbose) printf("Error @ [0x%8.8zx]:\n", err_addr);
             if (verbose) printf("\tInvalid NC component tag, while ABSENT is expected for ");
             DEBUG_RETURN_ERROR(NC_ENOTNC)
         }
@@ -2011,7 +2011,7 @@ val_NC_check_voff(NC *ncp)
 
     if (ncp->begin_rec < prev_off) {
         if (verbose) printf("Error:\n");
-        if (verbose) printf("\tRecord variable section begin offset (%lld) is less than fix-sized variable section end offset (%lld)\n", varp->begin, prev_off);
+        if (verbose) printf("\tRecord variable section begin offset (%lld) is less than fixed-size variable section end offset (%lld)\n", varp->begin, prev_off);
         nerrs++;
         DEBUG_ASSIGN_ERROR(status, NC_ENOTNC)
     }
@@ -2188,6 +2188,45 @@ val_get_NC(int fd, NC *ncp)
         goto fn_exit;
     }
 
+    /* check zero padding in the blank space betwee header size and extent */
+    if (repair && ncp->begin_var - ncp->xsz > 0) {
+        size_t i, gap = ncp->begin_var - ncp->xsz;
+        ssize_t readLen;
+        char *buf = (char*) malloc(gap);
+
+        if (-1 == lseek(fd, ncp->xsz, SEEK_SET)) {
+            if (verbose)
+                printf("Error at line %d: lseek %s\n",__LINE__,strerror(errno));
+            free(buf);
+            goto fn_exit;
+        }
+        readLen = read(fd, buf, gap);
+        if (readLen == -1) {
+            if (verbose)
+                printf("Error at line %d: read %s\n",__LINE__,strerror(errno));
+            free(buf);
+            status = -1;
+            goto fn_exit;
+        }
+        for (i=0; i<readLen; i++)
+            if (buf[i] != 0)
+                break;
+        if (readLen < gap || i < readLen) { /* zero out the blank space */
+            memset(buf, 0, gap);
+            if (lseek(fd, ncp->xsz, SEEK_SET) < 0) {
+                free(buf);
+                status = -1;
+                goto fn_exit;
+            }
+            if (write(fd, buf, gap) < 0) {
+                free(buf);
+                status = -1;
+                goto fn_exit;
+            }
+        }
+        free(buf);
+    }
+
 fn_exit:
     free(getbuf.base);
 
@@ -2196,6 +2235,7 @@ fn_exit:
 
 /* End Of get NC */
 
+#ifndef BUILD_CDFDIFF
 static void
 usage(char *argv0)
 {
@@ -2214,7 +2254,7 @@ int main(int argc, char **argv)
 {
     extern int optind;
     char filename[512], *path;
-    int i, omode, fd, status=NC_NOERR;
+    int i, omode, fd, fmt=1, status=NC_NOERR;
     NC *ncp=NULL;
     struct stat ncfilestat;
 
@@ -2273,6 +2313,9 @@ int main(int argc, char **argv)
     if (status != NC_NOERR && status != NC_ENULLPAD && status != -1)
         goto prog_exit;
 
+    /* class file format: CDF-1, 2 or 5 */
+    fmt = ncp->format;
+
     /* check data size */
     if (-1 == fstat(fd, &ncfilestat)) {
         if (verbose) printf("Error at line %d fstat (%s)\n",__LINE__,strerror(errno));
@@ -2303,7 +2346,7 @@ int main(int argc, char **argv)
         if (ncp->vars.ndefined == 0)
             expect_fsize = ncp->xsz;
         else
-            /* find the size of last fix-sized variable */
+            /* find the size of last fixed-size variable */
             expect_fsize = ncp->vars.value[ncp->vars.ndefined-1]->begin +
                            ncp->vars.value[ncp->vars.ndefined-1]->len;
         if (expect_fsize < ncfilestat.st_size) {
@@ -2332,9 +2375,9 @@ prog_exit:
 
     if (verbose) {
         if (status == NC_NOERR)
-            printf("File \"%s\" is a valid NetCDF classic file.\n",filename);
+            printf("File \"%s\" is a valid NetCDF classic CDF-%d file.\n",filename, fmt);
         else {
-            printf("File \"%s\" fails to conform with classic CDF file format specifications\n",filename);
+            printf("File \"%s\" fails to conform with classic CDF-%d file format specifications\n",filename, fmt);
             if (repair) {
                 printf("and it has been repaired in place to remove the errors.\n");
                 printf("Please run \"%s %s\" to validate again.\n",argv[0],filename);
@@ -2344,3 +2387,4 @@ prog_exit:
 
     exit((status == NC_NOERR) ? EXIT_SUCCESS : EXIT_FAILURE);
 }
+#endif
