@@ -61,6 +61,7 @@ static float range[] = {0., 1500.};
 static float lats[NLATS] = {-90, -87.5, -85, -82.5};
 static float lons[NLONS] = {-180, -175, -170};
 static int frtimes[NFRTIMES] = {12, 18};
+static int iscalar = 99;
 static const char* s = "1992-3-21 12:00";
 static float fill_value = -9999.0;
 static float P_data[NFRTIMES][NLATS][NLONS] = {
@@ -318,6 +319,8 @@ int gen(const MPI_Comm        &comm,
         reftime.putVar_all(str);
         free(str);
 
+        scalar.putVar_all(&iscalar);
+
         // we write one record at a time
         vector<MPI_Offset> startp,countp;
         startp.push_back(0);
@@ -492,8 +495,12 @@ void dump(const MPI_Comm &comm, const char* path)
     cout << "}" << endl;
 }
 
-/* Test everything for classic, 64-bit offset, an 64-bit data files. */
+/* Test everything for classic, 64-bit offset, 64-bit data, and netCDF4 files. */
+#ifdef ENABLE_NETCDF4
+#define NUM_FORMATS (4)
+#else
 #define NUM_FORMATS (3)
+#endif
 
 int
 main(int argc, char* argv[])	// test new netCDF interface
@@ -522,7 +529,12 @@ main(int argc, char* argv[])	// test new netCDF interface
 
    // Set up the format constants.
    NcmpiFile::FileFormat format[NUM_FORMATS] =
-              {NcmpiFile::classic, NcmpiFile::classic2, NcmpiFile::classic5};
+#ifdef ENABLE_NETCDF4
+       {NcmpiFile::classic, NcmpiFile::classic2, NcmpiFile::nc4, NcmpiFile::classic5};
+#else
+       {NcmpiFile::classic, NcmpiFile::classic2, NcmpiFile::classic5};
+#endif
+
 #ifdef DEBUG
    char format_name[NUM_FORMATS][NC_MAX_NAME] =
         {"classic", "classic2", "classic5"};
