@@ -32,14 +32,15 @@ int main(int argc, char **argv) {
 
     if (rank == 0) {
         char *cmd_str = (char*)malloc(strlen(argv[0]) + 256);
-        sprintf(cmd_str, "*** TESTING C   %s for inquiring CDF file formats ", basename(argv[0]));
+        sprintf(cmd_str, "*** TESTING C   %s for inquiring file formats ", basename(argv[0]));
         printf("%-66s ------ ", cmd_str);
         free(cmd_str);
     }
 
     /* test CDF-1 -----------------------------------------------------------*/
     sprintf(filename,"%s/test_cdf1.nc",dir_name);
-    err = ncmpi_open(MPI_COMM_WORLD, filename, 0, MPI_INFO_NULL, &ncid); CHECK_ERR
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL,
+                     &ncid); CHECK_ERR
 
     /* test NULL argument */
     err = ncmpi_inq_format(ncid, NULL); CHECK_ERR
@@ -64,7 +65,8 @@ int main(int argc, char **argv) {
 
     /* test CDF-2 -----------------------------------------------------------*/
     sprintf(filename,"%s/test_cdf2.nc",dir_name);
-    err = ncmpi_open(MPI_COMM_WORLD, filename, 0, MPI_INFO_NULL, &ncid); CHECK_ERR
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL,
+                     &ncid); CHECK_ERR
 
     /* test NULL argument */
     err = ncmpi_inq_format(ncid, NULL); CHECK_ERR
@@ -86,7 +88,8 @@ int main(int argc, char **argv) {
 
     /* test CDF-5 -----------------------------------------------------------*/
     sprintf(filename,"%s/test_cdf5.nc",dir_name);
-    err = ncmpi_open(MPI_COMM_WORLD, filename, 0, MPI_INFO_NULL, &ncid); CHECK_ERR
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL,
+                     &ncid); CHECK_ERR
 
     /* test NULL argument */
     err = ncmpi_inq_format(ncid, NULL); CHECK_ERR
@@ -106,6 +109,36 @@ int main(int argc, char **argv) {
         nerrs++;
     }
 
+    /* test NetCDF4 --------------------------------------------------------*/
+    sprintf(filename,"%s/test_netcdf4.nc",dir_name);
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL, &ncid);
+    if (PNETCDF_DRIVER_NETCDF4 == 0)
+        EXP_ERR(NC_ENOTBUILT)
+    else { /* NetCDF-4 is enabled */
+        CHECK_ERR
+
+        /* test NULL argument */
+        err = ncmpi_inq_format(ncid, NULL); CHECK_ERR
+
+        err = ncmpi_inq_format(ncid, &format); CHECK_ERR
+        if (format != NC_FORMAT_NETCDF4) {
+            printf("Error at line %d in %s: expecting NetCDF-4 format for file %s but got %d\n",
+                   __LINE__,__FILE__,filename,format);
+            nerrs++;
+        }
+        err = ncmpi_close(ncid); CHECK_ERR
+
+        /* test NULL argument */
+        err = ncmpi_inq_file_format(filename, NULL); CHECK_ERR
+
+        err = ncmpi_inq_file_format(filename, &format); CHECK_ERR
+        if (format != NC_FORMAT_NETCDF4) {
+            printf("Error at line %d in %s: expecting NETCDF4 format for file %s but got %d\n",
+                   __LINE__,__FILE__,filename,format);
+            nerrs++;
+        }
+    }
+
     MPI_Offset malloc_size, sum_size;
     err = ncmpi_inq_malloc_size(&malloc_size);
     if (err == NC_NOERR) {
