@@ -34,9 +34,8 @@ int main(int argc, char** argv)
     int ncid, cmode, varid, dimid[2], req[3], st[3];
     MPI_Offset start[2], count[2];
     MPI_Info info;
-#ifdef ENABLE_LARGE_REQ
     size_t i;
-#endif
+    int bb_enabled=0;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -61,7 +60,7 @@ int main(int argc, char** argv)
 
     buf = (unsigned char*) calloc(TWO_G+1024,1);
     if (buf == NULL) {
-        printf("malloc falled for size %lld\n", TWO_G+1024);
+        printf("malloc failed for size %lld\n", TWO_G+1024);
         MPI_Finalize();
         return 1;
     }
@@ -70,16 +69,19 @@ int main(int argc, char** argv)
     MPI_Info_set(info, "romio_cb_write", "enable");
     MPI_Info_set(info, "romio_ds_read", "disable"); /* run slow without it */
 
-#ifndef ENABLE_LARGE_REQ
+#if defined(ENABLE_LARGE_SINGLE_REQ) || defined(ENABLE_BURST_BUFFER)
+#else
     /* silence iternal debug messages */
     setenv("PNETCDF_SAFE_MODE", "0", 1);
 #endif
 
+#ifdef ENABLE_NETCDF4
+    /* Test for NetCDF 4 first as ncvalidator checks only read classic files */
+
     /* create a new file for writing ----------------------------------------*/
-    cmode = NC_CLOBBER | NC_64BIT_DATA;
+    cmode = NC_CLOBBER | NC_NETCDF4;
     err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, info, &ncid);
     CHECK_ERR
-    MPI_Info_free(&info);
 
     /* define dimensions */
     err = ncmpi_def_dim(ncid, "NPROCS", nprocs, &dimid[0]);
@@ -97,9 +99,97 @@ int main(int argc, char** argv)
     CHECK_ERR
 
     /* now we are in data mode */
-#ifdef ENABLE_LARGE_REQ
     for (i=0; i<20; i++) buf[ONE_G-10+i] = 'a'+i;
     for (i=0; i<20; i++) buf[TWO_G-10+i] = 'A'+i;
+
+    start[0] = rank;
+    count[0] = 1;
+
+    start[1] = 0;
+    count[1] = 10;
+    err = ncmpi_put_vara_uchar_all(ncid, varid, start, count, buf);
+    CHECK_ERR
+
+    /* 2nd request is not contiguous from the first */
+    start[1] = 1024;
+    count[1] = ONE_G-1024;
+    err = ncmpi_put_vara_uchar_all(ncid, varid, start, count, buf+1024);
+    CHECK_ERR
+
+    /* make file access and write buffer of 3rd request contiguous from the 2nd
+     * request to check whether the internal fileview and buftype coalescing
+     * are skipped */
+    start[1] = ONE_G;
+    count[1] = ONE_G+1024;
+    err = ncmpi_put_vara_uchar_all(ncid, varid, start, count, buf+ONE_G);
+    CHECK_ERR
+
+    start[1] = 0;
+    count[1] = 10;
+    err = ncmpi_get_vara_uchar_all(ncid, varid, start, count, buf);
+    CHECK_ERR
+
+    start[1] = 1024;
+    count[1] = ONE_G-1024;
+    err = ncmpi_get_vara_uchar_all(ncid, varid, start, count, buf+1024);
+    CHECK_ERR
+
+    start[1] = ONE_G;
+    count[1] = ONE_G+1024;
+    err = ncmpi_get_vara_uchar_all(ncid, varid, start, count, buf+ONE_G);
+    CHECK_ERR
+
+    err = ncmpi_close(ncid); CHECK_ERR
+
+    /* check if open to read header fine */
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL, &ncid); CHECK_ERR
+    err = ncmpi_close(ncid); CHECK_ERR
+#endif
+    /* Test classic format */
+
+    /* create a new file for writing ----------------------------------------*/
+    cmode = NC_CLOBBER | NC_64BIT_DATA;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, info, &ncid);
+    CHECK_ERR
+    MPI_Info_free(&info);
+
+    {
+        int flag;
+        char hint[MPI_MAX_INFO_VAL];
+        MPI_Info infoused;
+
+        ncmpi_inq_file_info(ncid, &infoused);
+        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
+        if (flag && strcasecmp(hint, "enable") == 0)
+            bb_enabled = 1;
+        MPI_Info_free(&infoused);
+    }
+
+    /* define dimensions */
+    err = ncmpi_def_dim(ncid, "NPROCS", nprocs, &dimid[0]);
+    CHECK_ERR
+
+    err = ncmpi_def_dim(ncid, "X", TWO_G+1024, &dimid[1]);
+    CHECK_ERR
+
+    /* define a big 1D variable of ubyte type */
+    err = ncmpi_def_var(ncid, "big_var", NC_UBYTE, 2, dimid, &varid);
+    CHECK_ERR
+
+    /* do not forget to exit define mode */
+    err = ncmpi_enddef(ncid);
+    CHECK_ERR
+
+    /* now we are in data mode */
+#if defined(ENABLE_LARGE_SINGLE_REQ) || defined(ENABLE_BURST_BUFFER)
+#ifndef ENABLE_LARGE_SINGLE_REQ
+    if (bb_enabled) {
+#endif
+    for (i=0; i<20; i++) buf[ONE_G-10+i] = 'a'+i;
+    for (i=0; i<20; i++) buf[TWO_G-10+i] = 'A'+i;
+#ifndef ENABLE_LARGE_SINGLE_REQ
+    }
+#endif
 #endif
 
     start[0] = rank;
@@ -125,9 +215,10 @@ int main(int argc, char** argv)
     CHECK_ERR
 
     err = ncmpi_wait_all(ncid, 3, req, st);
-#ifndef ENABLE_LARGE_REQ
-    EXP_ERR(NC_EMAX_REQ)
-#else
+#if defined(ENABLE_LARGE_SINGLE_REQ) || defined(ENABLE_BURST_BUFFER)
+#ifndef ENABLE_LARGE_SINGLE_REQ
+    if (bb_enabled) {
+#endif
     CHECK_ERR
 
     /* read back to check contents */
@@ -158,6 +249,13 @@ int main(int argc, char** argv)
 
     /* test the same pattern but for iget */
     for (i=0; i<TWO_G+1024; i++) buf[i] = 0;
+#ifndef ENABLE_LARGE_SINGLE_REQ
+    }
+    else
+        EXP_ERR(NC_EMAX_REQ)
+#endif
+#else
+    EXP_ERR(NC_EMAX_REQ)
 #endif
 
     start[1] = 0;
@@ -176,7 +274,7 @@ int main(int argc, char** argv)
     CHECK_ERR
 
     err = ncmpi_wait_all(ncid, 3, req, st);
-#ifndef ENABLE_LARGE_REQ
+#ifndef ENABLE_LARGE_SINGLE_REQ
     EXP_ERR(NC_EMAX_REQ)
 #else
     CHECK_ERR
@@ -198,8 +296,11 @@ int main(int argc, char** argv)
     }
 #endif
 
-    err = ncmpi_close(ncid);
-    CHECK_ERR
+    err = ncmpi_close(ncid); CHECK_ERR
+
+    /* check if open for reading header */
+    err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL, &ncid); CHECK_ERR
+    err = ncmpi_close(ncid); CHECK_ERR
 
     free(buf);
 
