@@ -49,7 +49,8 @@ inRange_uchar(const int     cdf_format,
 {
     /* check value of type xtype if within uchar range */
 
-    if (cdf_format < NC_FORMAT_CDF5 && xtype == NC_BYTE) {
+    if ((cdf_format <= NC_FORMAT_CDF2 ||
+         cdf_format == NC_FORMAT_NETCDF4_CLASSIC) && xtype == NC_BYTE) {
         /* netCDF specification make a special case for type conversion between
          * uchar and NC_BYTE: do not check for range error. See
          * http://www.unidata.ucar.edu/software/netcdf/docs/data_type.html#type_conversion
@@ -569,7 +570,8 @@ hash4(const int         cdf_format,
      * unsigned. Thus, no NC_ERANGE error can occur converting between NC_BYTE
      * and unsigned char.
      */
-    if (cdf_format < NC_FORMAT_CDF5 &&
+    if ((cdf_format <= NC_FORMAT_CDF2 ||
+         cdf_format == NC_FORMAT_NETCDF4_CLASSIC) &&
         itype == NCT_UCHAR && xtype == NC_BYTE &&
         result >= -128 && result < 0)
         result += 256;
@@ -806,7 +808,9 @@ put_vars(int ncid, int numVars)
         char hint[MPI_MAX_INFO_VAL];
         MPI_Info infoused;
 
-        ncmpi_inq_file_info(ncid, &infoused);
+        err = ncmpi_inq_file_info(ncid, &infoused);
+        IF (err != NC_NOERR) error("ncmpi_inq_file_info");
+
         MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
         if (flag && strcasecmp(hint, "enable") == 0)
             bb_enabled = 1;
@@ -852,6 +856,10 @@ put_vars(int ncid, int numVars)
                  * http://www.unidata.ucar.edu/software/netcdf/docs/group__error.html
                  */
                 if (bb_enabled){
+                    /* when using burst buffering, NC_ERANGE is reported
+                       at the flushing time */
+                    IF (err != NC_NOERR)
+                        error("ncmpi_put_vara_double_all: %s", ncmpi_strerror(err));
                     err = ncmpi_flush(ncid);
                 }
                 IF (err != NC_ERANGE)
@@ -906,7 +914,6 @@ check_dims(int  ncid)
     }
 }
 
-
 /*
  * check variables of specified file have expected name, type, shape & values
  */
@@ -1053,7 +1060,7 @@ check_file(char *filename, int numGatts, int numVars)
         check_dims(ncid);
         check_vars(ncid, numVars);
         check_atts(ncid, numGatts, numVars);
-        err = ncmpi_close (ncid);
+        err = ncmpi_close(ncid);
         IF (err != NC_NOERR)
             error("ncmpi_close: %s", ncmpi_strerror(err));
     }
@@ -1080,7 +1087,7 @@ s_nc_type(nc_type xtype)
 }
 
 int
-nctypelen(nc_type xtype)
+sizeof_nctype(nc_type xtype)
 {
     switch(xtype){
         case NC_BYTE :
